# O Número Mágico Sete, Mais ou Menos Dois

"[The magical number](https://en.wikipedia.org/wiki/The_Magical_Number_Seven,_Plus_or_Minus_Two)" 
(O Número Mágico Sete, Mais Ou Menos Dois) é um artigo sobre a psicologia do
número de coisas que nós conseguimos manter na nossa memória ao mesmo tempo.

Pelo menos duas vezes eu vi esse tipo de construção onde uma função faz um
processamento, mais o retorno é o resultado desse processamento, mais o
resultado de uma segunda função e um processamento. Nada muito grande. Mas a
segunda função também faz um processamento e chama uma terceira função. E a
terceira função chama uma quarta. E a quarta, uma quinta. E a quinta função
chama uma sexta.

E "processamento" não era nada pequeno, como "adicionar dois" ou "multiplicar
por si mesmo ou por um valor configurável".

Algo do tipo:

```
func_1
 +-- func_2
     +-- func_3
	     +-- func_4
		     +-- func_5
			     +-- func6
```

(Se você estiver curioso sobre o verdadeiro processamento que eu vi, era uma
classe com uma função que fazia um processamento e chamava uma função de uma
classe injetada com injeção de dependências -- que é bem legal. Mas a
classe/dependência injetada tinha uma dependência injetada, e a terceira
injeção de dependência _também_ tinha uma dependência injetada e assim por
diante.)

Agora, quando você está tentando entender esse tipo de código para encontrar
um problema, você precisa manter em mente o que a primeira, segunda, terceira,
quarta, quinta e sexta funções fazem, porque elas ficam se chamando
(internamente).

Isso causa uma sobrecarga mental que não deveria ser necessária.

Não apenas isso, mas imagine que você bote um log antes e depois de chamar
`func_1`: o primeiro log vai mostrar os dados antes de serem enviados para a
função e o segundo log vai mostrar o resultado da chamada.

Aí você vai ficar com a impressão que `func_1` faz um monte de coisas, sendo
que ela faz uma transformação simples e passa adiante (novamente,
internamente).

(Eu tive essa experiência estranha com uma função chamada `expand`, que o log
antes da chamada mostrar os dados crus, compactado, mas o resultado não era
era o resultado descompactado, mas os dados já processados depois de
compactados.)

E qual seria a solução pra isso, você deve estar perguntando.

Bom, se ao invés de fazer a `func_1` chamada a `func_2`, você pode fazer ela
retornar o resultado (que não necessariamente é o resultado final) e _então_
chamar `func_2` com esse resultado.

Algo do tipo:

```
result1 = func_1
result2 = func_2(result1)
result3 = func_3(result2)
result4 = func_4(result3)
result5 = func_5(result4)
result6 = func_6(result5)
result7 = func_7(result6)
```

(Se voltarmos a nossa cadeira de injeção de dependências, você pode imaginar
quao invés de fazer DI7[^1] receber DI6 [que iria receber DI5 como
dependência, que iria receber DI4 como dependência, que iria receber DI3 como
dependência e assim por diante], você pode receber todas as dependências
injetadas numa única passada e aí iniciar a função que iria chamar as
dependência num só tacada, ao invés de encadear uma na outra.)

Agora você consegue entender _exatamente_ como é que os dados são
transformados -- e as funções teriam nomes melhores, como `expand`,
`break_lines`, `name_fields` e assim por diante, de forma que você consiga ver
que os dados compactados que eu comentei anteriormente são descompactados, o
conteúdo é quebrado linha por linha, e as linhas estão sendo tratadas para
colocar os nomes nos campos e assim por diante (e alguém poderia até mesmo
dizer que as coisas ficariam mais claras se depois de `break_lines` tivesse
uma função chamada `break_fields`, o que faria a função de `name_fields` mais
óbvia -- e uma construção como essa faria com que essa alteração fosse
trivial).

"Não isso não performa!" alguém pode gritar. Bom, talvez seja um pouco menos
performático que o original com chamadas aninhadas (porque não precisaria
criar e destruir os frames na stack, simplesmente iria empilhar os frames e
destruir tudo no final), mas otimização é para compiladores, não pessoas. O
seu trabalho é fazer com que o código seja _legível_ e _inteligível_. Se você
precisa de performance, você precisa pensar numa melhor sequência de passos
(ou um armazenamento de dados melhor), não uma solução terrível de ser
entendida.

Só uma nota rápida: Embora o artigo acima indique o número é próximo de 7,
novas pesquisas mostram que esse número é bem mais baixo que isso, mais para 4.
Então simplesmente fazendo `func_1` chamar `func_2`, que chamaria `func_3`,
que chamaria `func_4` já seria o suficiente para sobrecarregar alguém e fazer
com que eles percam a ideia do que o código faz.

PS: Se você conhecer um pouco de programação funcional, eu estou sugerindo
usar [composição de
funções](https://pt.wikipedia.org/wiki/Composi%C3%A7%C3%A3o_de_fun%C3%A7%C3%B5es)
ao invés de fazer chamadas aninhadas.

---

[^1]: A classe no sétimo nível de injeção de dependência; a primeira classe
  com dependências injetadas seria "DI1".

<!-- 
vim:spelllang=pt:
-->
