# Aprenda O Básico de Programação Funcional

Nessa altura do campeonato, você deve ter ouvido falar de como programação
funcional é legal. Existem vários conceitos nela, mas tenha em mente os
conceitos mais básicos dela.

Um monte de apresentações sobre programação funcional vêm com palavras
estranhas como "functors" e "monads". Não que seja prejudicial conhecer o que
elas querem dizer (aviso: eu ainda não sei). Mas algumas coisas da programação
funcional são fáceis de entender e usar.

Por exemplo, imutabilidade. Isso quer dizer que os seus dados não podem mudar
depois de criados. Você tem um registro de um usuário e o usuário mudou de
senha? Não, não mude o valor do campo de senha, crie um novo registro de
usuário com a senha atualizada e remova o antigo. Eu sei que isso cria uma
sequencia de "cria e destrói" que basicamente não fazem sentido (porque você
deveria alocar memória para um novo usuário, copiar todos os campos com exceção
de um, definir apenas um campo, e liberar a memória do antigo? Não faz
sentido!) mas, a longo prazo, isso previne resultados estranhos, principalmente
depois de você entender e começar a usar threads.

(A ideia é prevenir um estado compartilhado -- memória -- entre partes do seu
código.)

Outro conceito útil são funções puras. Funções puras são funções que, se
chamadas com os mesmos parâmetros, sempre retornam o mesmo resultado, não
importa quantas vezes você as chame. Um exemplo de função não pura é o
`random()`: cada vez que você chama `random()`, você tem como retorno um valor
diferente[^1]. Um exemplo de uma função pura seria algo parecido com isso em
Python:

```python
def mult(x):
    return x * 4
```

Não importa quantas vezes você chame `mult2)`, sempre terá o 8 como resultado.

Outro exemplo seria nossa senha imutável apresentada acima: Você pode
facilmente escrever uma função que recebe um registro de usuário e retorna um
novo registro com a senha alterada. Você pode chamar a função várias vezes e
ainda ter o mesmo resultado no final.

Funções puras são úteis porque, principalmente, elas são fáceis de serem
testadas.

Segundo, elas são fáceis de serem encadeadas num [fluxo de
dados](./fluxo-dedados.md): Como elas não tem estado interno (que é o
verdadeiro motivo de serem chamadas funções puras), você pode facilmente chamar
uma depois da outra e não importa quantas vezes você passe valores entre elas,
elas sempre produzem o mesmo resultado. E como cada função, dado a mesma
entrada, produz o mesmo resultado, encadeando todas elas _também_ gera o mesmo
resultado.

Só esses dois conceitos podem fazer com que você tenha mais código (de novo,
você está criando um novo registro ao invés de simplesmente alterar apenas um
campo), mas o resultado final é que o seu código ficará mais robusto.

[^1]: Com exceção de Haskell, mas ela requer que você envie uma semente toda
  vez, fazendo com que você tenha valores baseados nessa semente, e assim a
  função ainda é pura.

<!--
vim:spelllang=pt:
-->
